//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : ConsoleDemo.cpp
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1998, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// This is a simple demo that uses the CVMEditor library from a
// console mode application.
//
// Syntax: ConsoleDemo filename.txt [search_string [replace_string]]
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    04/15/98  EFW   Created the code
//=============================================================================

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <conio.h>
#include <stdio.h>

#include "CVMEditor.h"
#include "AutoLink.h"

BOOL bSearchDone = FALSE;
int  nFindCount = 0;

// A simple editorDialog() handler
int doEditorDialog(int nCode, ...)
{
    switch(nCode)
    {
        case edSearchFailed:
            if(!nFindCount)
            {
                printf("\n\nSearch string not found.  "
                    "Press any key to continue\n");
                getch();
            }
            bSearchDone = TRUE;
            break;

        default:
            printf("\n\nCVMEditor Code: %d\nPress any key to continue...",
                nCode);

            getch();
            break;
    }

    return IDCANCEL;
}

// A simple signsOfLife() handler
BOOL doSignsOfLife(SignsOfLife lifeSign, UINT cmnd, long count,
    long total)
{
    switch(lifeSign)
    {
        case slReporting:
            printf("Processing %ld of %ld\r", count, total);
            break;

        case slDone:
            printf("                                                  \r");
            break;

        default:
            break;
    }

    return TRUE;
}

int main(int argc, char *argv[])
{
    int  x;
    long y, line;

    if(argc < 2)
    {
        printf("ConsoleDemo filename.txt [search_string [replace_string]]\n");
        return 1;
    }

    CVMEditor::editorDialog = doEditorDialog;
    CVMEditor::signsOfLife = doSignsOfLife;

    CVMEditor editor(20, argv[1]);
    editor.textLimits(x, y);

    switch(argc)
    {
        case 3:
            // Search for a text string and show all lines
            // that contain it.
            bSearchDone = FALSE;
            strcpy(editor.szFindStr, argv[2]);
            editor.nSearchOrigin = efEntireScope;

            while(!bSearchDone)
            {
                editor.doSearchReplace();

                // If there was a find, show the line
                if(!bSearchDone)
                {
                    printf("%s\n", editor.lineText());

                    // After the first find, switch to From Cursor mode
                    editor.nSearchOrigin = efFromCursor;
                    nFindCount++;
                }
            }

            printf("\n\nFound %d occurances\n", nFindCount);
            break;

        case 4:
            // Search for a string and replace it with another
            CVMEditor::GlobalOpts.nSearch |= (efDoReplace | efReplaceAll);
            CVMEditor::GlobalOpts.nSearch &= ~efPromptOnReplace;
            editor.nSearchOrigin = efEntireScope;

            strcpy(editor.szFindStr, argv[2]);
            strcpy(editor.szReplaceStr, argv[3]);

            nFindCount = editor.doSearchReplace();
            printf("Found and replaced %d occurances\n", nFindCount);

            // Save the changed file if there where replacements.
            // This is just a demo so it will not alter the original
            // file.  It saves the file in the current directory
            // with a new name.
            if(nFindCount)
            {
                printf("Saving changes to _Test_.txt\n");

                // Save the changes to a new name
                editor.save("_Test_.txt");
            }
            break;

        default:
            // If not searching or replacing, just dump the file's contents
            printf("Contents of %s:\n\n", editor.Buffer->szFilename);

            for(line = 1; line <= y; line++)
                printf("%06ld: %s\n", line, editor.Buffer->getLine(line));

            break;
    }

    return 0;
}
