//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : CVMEditorDefs.h
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1994-98, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// Supporting definitions and data structures.
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    06/14/97  EFW   Ported the code to Visual C++
//=============================================================================

#if !defined(__CVMEDITORDEFS_H)
#define __CVMEDITORDEFS_H

#define CVMEDITOR_VERSION   0x00001000     // 1.0.0

#if defined(SHAREWARE)
#define CVMEDITOR_VERSTRING "1.0.0 (Unregistered)"
#else
#define CVMEDITOR_VERSTRING "1.0.0"
#endif

#if !defined(EOS)
#define EOS     '\x0'
#endif

#define SZAUTOSAVE_STRING       "~~"        // Auto save modifier
#define SZAUTOSAVE_STRINGLEN    2           // Length of auto save modifier

// Keyboard command flag modifiers
const
  kfCtrlFlag  = 0x01,
  kfShiftFlag = 0x02,
  kfAltFlag   = 0x04;

// Update hint flags.  The bulk of the editor engine does not interact with
// the user interface at all.  In a number of places though, there had to be
// a way to notify the user interface that text had changed.  These flags
// along with the update() function allow this to happen in a generic way
// that is compatible with any UI library (MFC, OWL, etc).
const
  ufNone    = 0x00000000,   // No change/clear flags
  ufUpdate  = 0x00000001,   // Update attached indicators, scrollbars, etc
  ufLine    = 0x00000002,   // Update one or more lines
  ufView    = 0x00000004,   // Update all visible lines
  ufCenter  = 0x00000008,   // Center the cursor when next updated
  ufTrack   = 0x00000010,   // Ensure cursor is visible when next updated
  ufCaret   = 0x00000020,   // Insert mode or font changed, update caret size
  ufWrap    = 0x00000040,   // Redraw from cursor line on down after auto wrap
  ufReshare = 0x00000080;   // Reset shared settings

// Block marking mode indicators
const
  smNone     = 0x0000,      // No block selection in progress
  smBlock    = 0x0001,      // Block Key marking
  smShift    = 0x0002,      // Shift+Movement Key marking
  smMouse    = 0x0004;      // Mouse drag marking

// Editor dialog types
const
  edOutOfMemory   = 0,
  edReadError     = 1,
  edWriteError    = 2,
  edCreateError   = 3,
  edSwapBadSeek   = 4,
  edSwapFileFail  = 5,
  edGeneralMsg    = 6,      // Used by internal diagnostic messages

  edOpCancelled   = 7,      // The default editor dialog function
  edLinesSplit    = 8,      // will ignore dialog requests above
  edLineTooLong   = 9,      // edGeneralMsg and automatically return
  edGotoLine      = 10,     // IDCANCEL.  For edGeneralMsg or below,
  edChgLocalOpts  = 11,     // a simple message box is displayed to
  edFind          = 12,     // draw attention to the error.  They
  edReplace       = 13,     // are serious and should not be ignored.
  edReplacePrompt = 14,
  edSearchFailed  = 15,
  edReadBlock     = 16,
  edSaveBlock     = 17,
  edSaveAs        = 18,
  edSaveUntitled  = 19,
  edSaveModify    = 20,
  edAlterText     = 21,
  edReSynch       = 22,
  edLoadAutoSave  = 23;

// Editor option bit masks
const long                          // Local editor options
  efAutoIndent     = 0x00000001,    // Auto-indent mode on/off
  efWordWrap       = 0x00000002,    // Word wrap mode on/off
  efAutoWrap       = 0x00000004,    // Auto word wrap (alters efWordWrap usage)
  efWrapOnResize   = 0x00000008,    // Wrap on window resize on/off
  efTrailingSpaces = 0x00000010,    // Retain trailing spaces on/off
  efUseHardTabs    = 0x00000020,    // Use hard tabs on/off
  efCanUndo        = 0x00000040,    // Can use undo/redo

  efNonPersistentBlocks = 0x00000080,   // Blocks are non-persistent
  efAllowOverwrite      = 0x00000100,   // Overwrite non-persistent blocks
  efOverwriteBlocks     = 0x00000180,   // Single constant for checking whether
                                        // overwriteable blocks are allowed.

                                    // Global editor options
  efExpandTabs        = 0x00000200, // Expand tabs to spaces when loaded
  efBackupFiles       = 0x00000400, // Create backup file when saved
  efUseLFOnly         = 0x00000800, // LF only when file is saved (UNIX style)

  efResynchAppPrompt  = 0x00001000, // Prompt on app focus (app-specific)
  efResynchViewPrompt = 0x00002000, // Prompt on view focus (UI-specific)
  efResynchViewNever  = 0x00004000, // Never resynch with disk on view focus
  efResynchNever      = 0x00008000, // Never resynch with disk on any focus

                                    // Other options.  These bits are inherited
                                    // like local options but probably
                                    // shouldn't be user-modifiable.
  efReadOnly       = 0x00010000,    // File is Read Only and can't be edited
  efSetROonLoad    = 0x00020000,    // Set efReadOnly when file's RO
                                    // attribute is on

  efUseCSH         = 0x00040000;    // For use with Color Syntax Highlighter
                                    // Class Library.  The basic editor does
                                    // nothing with this.

// Enter Matching bit masks
const
  efParen          = 0x0001,    // Enter Matching character selections.
  efBracket        = 0x0002,    // These are local to each editor.
  efBrace          = 0x0004,
  efSngQuote       = 0x0008,
  efDblQuote       = 0x0010,
  efAngBracket     = 0x0020,
  efMatchAll       = 0x003F,    // Quick way to say do it for all of the above

  efAutoEOLMark    = 0x1000;    // Automatically insert EOLMarker on CVM_NEWLINE

// Search/Replace bit masks
const                           // Find/Replace settings are global options
  efCaseSensitive   = 0x0001,   // Find/Replace options
  efWholeWordsOnly  = 0x0002,
  efPromptOnReplace = 0x0004,

  efDoReplace       = 0x0008,   // Replace modes.  These three constants aren't
  efReplaceAll      = 0x0010,   // user-selectable modes, they are internal
  efReplaceOnly     = 0x0020;   // mode flags used by the editor.

const                           // More user-selectable Find/Replace options
  efForward      = 0x0000,      // Direction
  efBackward     = 0x0001,

  efGlobal       = 0x0000,      // Scope
  efSelectedText = 0x0001,

  efFromCursor   = 0x0000,      // Origin
  efEntireScope  = 0x0001;

// Other stuff
const
  maxLineLength = 256,          // Max line length
  maxSearchString = 80,         // Max search string length
  maxKeyMaps = 11;              // Maximum number of key maps

//*****************************************************************************

// Global editor options structure.  This structure contains options used
// by all editor object instances along with the defaults for the local
// options structure below.
struct GlobalEditorOptions
{
    long  lOperation;       // Default operating modes

    short nEnterMatch,      // Default Enter Matching characters
          nTabSize,         // Default tab size
          nIndentSize,      // Default block indent size
          nRightMargin;     // Default right margin

    // NOTE: Any changes in the structure ABOVE this point should also
    //       be taken into account for structure LocalEditorOptions below.
    //       The rest of this structure is for global items only.

    // Search options
    USHORT nSearch;

    // Signs of Life.
    // These two values indicate when signs of life should be issued by
    // the editor.  What this means is that after initThreshold lines,
    // the editor will place a call to the signsOfLife() function to let
    // the user know it is still working and hasn't locked up.  After that,
    // it will place a call to it every reportInterval lines.
    short nInitThreshold;
    short nReportInterval;

    // Default filename extensions
    char szDefaultExt[11], szBackupExt[11];
};

// Local editor options structure.  These options are local to each
// instance and can be changed for each one without affecting any of
// the other objects.  As above, changes here should be taken into
// acount for structure GlobalEditorOptions.
struct LocalEditorOptions
{
    long  lOperation;       // Operating modes

    short nEnterMatch,      // Enter Matching characters
          nTabSize,         // Tab size
          nIndentSize,      // Block indent size
          nRightMargin;     // Right margin
};

// Structure for getting search and replace options
struct FindReplaceStruct
{
    char szFindStr[maxSearchString];
    char szReplaceStr[maxSearchString];

    int  nOptions;      // efCaseSensitive, etc
    int  nDirection;
    int  nScope;
    int  nOrigin;
};

//*****************************************************************************

// Line end position used by the MeasureText() and reformatting functions
enum LineEndPos { epBeforeMargin, epAtMargin, epPastMargin };

// Editor dialog handler definitions
typedef int (*CVMEditorDialog)(int, ...);
int VMdefEditorDialog(int nDialog, ...);

// These are the different states that the signsOfLife() function can be
// called with to handle.
enum SignsOfLife { slInitiate, slReporting, slHide, slShow, slDone };

// Signs of Life handler definitions
typedef BOOL (*CVMSignsOfLife)(SignsOfLife, UINT, long, long);
BOOL VMdefSignsOfLife(SignsOfLife slLifeSign, UINT nCmnd, long lCount,
    long lTotal);

#endif
