//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : CVMPoint.h
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1994-98, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// This is the header file for the supporting CVMPoint class.  It
// encapsulates the row and column value for cursor positions within
// the text.
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    06/14/97  EFW   Ported the code to Visual C++
//=============================================================================

#if !defined(__CVMPOINT_H)
#define __CVMPOINT_H

// Class CVMPoint
//  Similar in nature to a regular point structure, but has a y value
//  that is long.  This allows the editor to load files with more than
//  32,767 lines in 16-bit implementations of the editor.
//
class CVMPoint
{
public:
    int   x;        // Column (limited to 0 to maxLineLength - 1)
    long  y;        // Line   (unlimited range)

    CVMPoint();
    CVMPoint(int nX, long lY);

    CVMPoint& operator+=( const CVMPoint& adder );
    CVMPoint& operator-=( const CVMPoint& subber );

    friend CVMPoint operator - ( const CVMPoint& one, const CVMPoint& two);
    friend CVMPoint operator + ( const CVMPoint& one, const CVMPoint& two);
    friend int operator == ( const CVMPoint& one, const CVMPoint& two);
    friend int operator != ( const CVMPoint& one, const CVMPoint& two);
};

inline CVMPoint::CVMPoint()
{
    x = 0;
    y = 0L;
}

inline CVMPoint::CVMPoint(int nX, long lY)
{
    x = nX;
    y = lY;
}

inline CVMPoint& CVMPoint::operator += ( const CVMPoint& adder )
{
    x += adder.x;
    y += adder.y;
    return *this;
}

inline CVMPoint& CVMPoint::operator -= ( const CVMPoint& subber )
{
    x -= subber.x;
    y -= subber.y;
    return *this;
}


#endif
