//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : MFCEditor.h
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1998, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// A CVMEditor-derived class that uses MFC.
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    06/08/97  EFW   Created the code
//=============================================================================

#if !defined(__MFCEDITOR_H)
#define __MFCEDITOR_H

#if !defined(__CVMEDITOR_H)
#include "CVMEditor.h"
#endif

// Draw mode color flags
#define DM_NORMAL_TEXT      0       // Normal
#define DM_HIGHLIGHT_TEXT   1       // Highlighted
#define DM_FINDLINE_TEXT    2       // For found text
#define DM_BOOKMARK         3       // For bookmarks
#define DM_GUTTER           4       // For the gutter

#define MAX_PRINT_PAGES     8192

/////////////////////////////////////////////////////////////////////////////
// A derived editor class suitable for use by a CView/CDocument pair or
// a CWnd-derived class used in a dialog box

class CMFCEditor : public CVMEditor
{
protected:
    CFont m_EditFont;

    HWND hWrapWnd;

    // Printing related members
    CFont m_PrinterFont;
    long *plBuffer;         // Pointer to the page index buffer
    CSize PrintSize;        // Char size for printing
    BOOL  bPrintSel;        // TRUE if printing the selected text
    int   nLinesPerPage;    // Number of lines per printed page
    int   nPtSize;          // Point size of the selected font

public:
    static int nDefGutter;  // Default size of view offset for gutter
    int   nGutterWidth;     // Actual width for this instance

    BOOL  bHasVertSB, bHasHorzSB;   // Scrollbar present flags

    CSize CharSize;         // Char size for display
    int   nTabWidth;        // Width of one tab character

    CMFCEditor(int bufSizeInK = 32, const char *pszFName = NULL,
        BOOL bSuppressLoad = FALSE);
    ~CMFCEditor();

    // Measure text for word wrapping
    virtual LineEndPos MeasureText(const char *pszLine, int nLen, int nCol,
        int *pnWrapCol = NULL);

    // Share a copy of the specified editor's buffer
    virtual void ShareBuffer(CMFCEditor *pEditor);

    // Set color for drawing
    virtual void setDrawColor(int nColorSel, CDC *pDC);

    // Draw view
    virtual void drawView(CDC *pDC, BOOL bIsFocusedView, CRect &rect);
    virtual void drawGutter(CDC &dc, CRect &rect);

    // Return indicator states in a text string
    virtual const char *getIndicatorInfo(char *pszIndBuf);

    // Printing functions
    virtual int  calcPrintedPages(CDC *pDC, BOOL bPrintSelection);
    virtual void printPage(CDC *pDC, UINT nPageNo);
    virtual void donePrinting(void);

    // Handle mouse-related block selection
    void calcCursorPos(CWnd *wnd, CPoint *point);
    void startMouseSelect(CWnd *wnd, CPoint *point);
    void dbclkMouseSelect(CPoint *point);
    BOOL moveMouseSelect(CWnd *wnd, CPoint *point);
    void endMouseSelect(CWnd *wnd, CPoint *point);

    // Update screen related stuff
    void updateView(CWnd *wnd);
    void getFontDimensions(CWnd *wnd);
    void getScreenDimensions(CWnd *wnd);
    void getFontInfo(LPLOGFONT lpFont);
    void setFontInfo(LPLOGFONT lpFont);
};

#endif
