//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : MFCEditorView.h
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1998, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// An MFC CView-derived class used to encapsulate a CMFCEditor object for
// use in a doc/view pair.
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    06/08/97  EFW   Created the code
//=============================================================================

#if !defined(__MFCEDITORVIEW_H)
#define __MFCEDITORVIEW_H

#if defined(USE_CSHLIB)

#if !defined(__MFCCSHEDITOR_H)
#include "MFCCSHEditor.h"
#endif

#else

#if !defined(__MFCEDITOR_H)
#include "MFCEditor.h"
#endif

#endif

class CMFCEditorView : public CView
{
protected: // create from serialization only
	CMFCEditorView();
	DECLARE_DYNCREATE(CMFCEditorView)

// Attributes
public:
	CMFCEditorDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMFCEditorView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMFCEditorView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

// Generated message map functions
protected:
	//{{AFX_MSG(CMFCEditorView)
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnEditcopy();
	afx_msg void OnEditcut();
	afx_msg void OnEditpaste();
	afx_msg void OnEditClear();
	afx_msg void OnEditSelectAll();
	afx_msg void OnUpdateClipCommand(CCmdUI* pCmdUI);
	afx_msg void OnUpdateUndoRedo(CCmdUI* pCmdUI);
	afx_msg void OnEditFind();
	afx_msg void OnEditRedo();
	afx_msg void OnEditRepeat();
	afx_msg void OnEditReplace();
	afx_msg void OnEditUndo();
	afx_msg void OnChangeFont();
    afx_msg BOOL OnMouseWheel(UINT fFlags, short zDelta, CPoint point);
	//}}AFX_MSG

    afx_msg void OnEditorCommands(UINT nCmd);
    afx_msg void OnUpdateIndicator(CCmdUI *pCmdUI);

	DECLARE_MESSAGE_MAP()

protected:
    int keyState;

    BOOL bIsResizing;
    CString csTwoKeyMsg, csLiteralMsg;

public:

#if defined(USE_CSHLIB)
    CMFCCSHEditor *editor;      // Color syntax highlighting editor
#else
    CMFCEditor *editor;         // Non-highlighting editor
#endif

    void WrapOnResize(BOOL bDone);
};

#ifndef _DEBUG  // debug version in MFCEditorView.cpp
inline CMFCEditorDoc* CMFCEditorView::GetDocument()
   { return (CMFCEditorDoc*)m_pDocument; }
#endif

#endif
