// MFCSDI.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "MFCSDI.h"

#include "CVMEditor.h"

#include "MFCEditorDoc.h"
#include "MFCEditorView.h"
#include "MFCEditorWnd.h"

#include "MainFrm.h"
#include "IniFile.h"
#include "GlobalOptions.h"
#include "SetLineDlg.h"
#include "MemoDemoDlg.h"
#include "ShowKeyMapDlg.h"

#if defined(USE_CSHLIB)
#include "MFCCSHEditor.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMFCSDIApp

BEGIN_MESSAGE_MAP(CMFCSDIApp, CWinApp)
	//{{AFX_MSG_MAP(CMFCSDIApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_OPTIONS_DEFAULTOPTS, OnOptionsDefaultopts)
	ON_COMMAND(ID_DEMO_TEXTSIZE, OnDemoTextsize)
	ON_COMMAND(ID_DEMO_SETLINEDEMO, OnDemoSetlinedemo)
	ON_COMMAND(ID_DEMO_DEFKEYMAP, OnDemoDefkeymap)
	ON_COMMAND(ID_DEMO_MEMODLG, OnDemoMemodlg)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMFCSDIApp construction

extern int doEditDialog(int dialog, ...);
extern BOOL doSignsOfLife(SignsOfLife lifeSign, UINT cmnd, long count,
    long total);

CMFCSDIApp::CMFCSDIApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMFCSDIApp object

CMFCSDIApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMFCSDIApp initialization

BOOL CMFCSDIApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

    SetRegistryKey("Eric Woodruff");

    // Make sure CVMEditorWndClass exists.  This only needs to be
    // done once.  It can be placed in the application's startup code
    // if CMFCEditorWnd is used in more than one dialog box
    CMFCEditorWnd::RegisterControlClass();

    // Set default dialog and signs of life handlers for the editor
    CVMEditor::editorDialog = doEditDialog;
    CVMEditor::signsOfLife = doSignsOfLife;

    // Memo demo text
    m_csFirstMemo = "First memo field\r\n\r\n"
        "This memo field handles both TAB and RETURN key presses.\r\n"
        "Press ^QO in either field to change the local options.\r\n\r\n"
        "It also has a gutter enabled to show bookmark tags.";

    m_csSecondMemo = "Second memo field\r\n\r\n"
        "This memo field handles neither TAB nor RETURN key presses.\r\n"
        "\r\nBecause there is no horizontal scrollbar, automatic word "
        "wrapping is\r\nenabled by default.\r\n\r\n"
        "The gutter has been left off of this field.";

    // Load standard INI file options (including MRU)
	LoadStdProfileSettings();

    // Load other INI settings
    CMFCEditor::nDefGutter = GetProfileInt(SEC_SETTINGS, INI_SHOWGUTTER, 1);
    if(CMFCEditor::nDefGutter)
        CMFCEditor::nDefGutter = DEF_GUTTER_SIZE;

    // Enable a visible Auto EOL marker?
    int nUseAutoEOL = GetProfileInt(SEC_SETTINGS, INI_USEAUTOEOL, NAUTOEOL);
    if(nUseAutoEOL)
    {
        CVMEditor::GlobalOpts.nEnterMatch |= efAutoEOLMark;
        CVMEditor::chEOLMarker = '\x07';
    }

    CMFCEditorDoc::nBufSize = GetProfileInt(SEC_SETTINGS, INI_EDITBUFSIZE,
        NEDITBUFSIZE);
    CVirtualMemory::setMaxAllocation(GetProfileInt(SEC_SETTINGS,
        INI_MAXALLOCATE, NMAXALLOCATE));

    // Global editor options
    CString csIniString;
    CVMEditor::GlobalOpts.lOperation = GetProfileInt(SEC_SETTINGS,
        INI_GBLOPTS, CVMEditor::GlobalOpts.lOperation);
    CVMEditor::GlobalOpts.nEnterMatch = GetProfileInt(SEC_SETTINGS,
        INI_ENTMATCH, CVMEditor::GlobalOpts.nEnterMatch);
    CVMEditor::GlobalOpts.nTabSize = GetProfileInt(SEC_SETTINGS,
        INI_TABSIZE, CVMEditor::GlobalOpts.nTabSize);
    CVMEditor::GlobalOpts.nIndentSize = GetProfileInt(SEC_SETTINGS,
        INI_INDENTSIZE, CVMEditor::GlobalOpts.nIndentSize);
    CVMEditor::GlobalOpts.nRightMargin = GetProfileInt(SEC_SETTINGS,
        INI_RIGHTMARGIN, CVMEditor::GlobalOpts.nRightMargin);
    CVMEditor::GlobalOpts.nSearch = GetProfileInt(SEC_SETTINGS,
        INI_SEARCHOPTS, CVMEditor::GlobalOpts.nSearch);
    CVMEditor::GlobalOpts.nInitThreshold = GetProfileInt(SEC_SETTINGS,
        INI_SLSTART, CVMEditor::GlobalOpts.nInitThreshold);
    CVMEditor::GlobalOpts.nReportInterval = GetProfileInt(SEC_SETTINGS,
        INI_SLINTERVAL, CVMEditor::GlobalOpts.nReportInterval);

    csIniString = GetProfileString(SEC_SETTINGS, INI_DEFEXT,
        CVMEditor::GlobalOpts.szDefaultExt);
    if(csIniString.GetLength() < 11)
        strcpy(CVMEditor::GlobalOpts.szDefaultExt, csIniString);

    csIniString = GetProfileString(SEC_SETTINGS, INI_BKEXT,
        CVMEditor::GlobalOpts.szBackupExt);
    if(csIniString.GetLength() < 11)
        strcpy(CVMEditor::GlobalOpts.szBackupExt, csIniString);

#if defined(USE_CSHLIB)
    // TO DO: Load name from registry
    static char szModuleName[MAX_PATH];
    char *pModulePath;

    szModuleName[0] = '\x0';
    GetModuleFileName(NULL, szModuleName, MAX_PATH);
    pModulePath = strrchr(szModuleName, '\\');
    if(!pModulePath)
        pModulePath = szModuleName;
    else
        pModulePath++;
    strcpy(pModulePath, "CSHDefaults.dta");

    CMFCCSHEditor::CSHDataSetName = szModuleName;
#endif

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CMFCEditorDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CMFCEditorView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

    // Initialize auto save
    ((CMainFrame *)m_pMainWnd)->nAutoSaveFlags = GetProfileInt(SEC_SETTINGS,
        INI_AUTOSAVEFLAGS, 0);
    ((CMainFrame *)m_pMainWnd)->nAutoSaveInterval = GetProfileInt(SEC_SETTINGS,
        INI_AUTOSAVEINTERVAL, asDefaultInterval);

    ((CMainFrame *)m_pMainWnd)->SetAutoSave(TRUE);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CStatic	m_scVersion;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_ABOUT_VERSION, m_scVersion);
	//}}AFX_DATA_MAP
}

BOOL CAboutDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

    CString csVer;
    csVer.Format("Version: %s", CVMEDITOR_VERSTRING);
    m_scVersion.SetWindowText(csVer);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CMFCSDIApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CMFCSDIApp commands


void CMFCSDIApp::OnOptionsDefaultopts()
{
	CGlobalOptions optsDlg(IDS_GLOBAL_OPTIONS);

    CMainFrame *pWnd = (CMainFrame *)m_pMainWnd;

    optsDlg.SetOptions(CVMEditor::GlobalOpts, pWnd->nAutoSaveFlags,
        pWnd->nAutoSaveInterval);

    if(optsDlg.DoModal() == IDOK)
    {
        // NOTE:  For SDI, most of these options will not take
        //        effect until the application is restarted.
        //        The same goes for open MDI editor windows.
        optsDlg.GetOptions(CVMEditor::GlobalOpts, pWnd->nAutoSaveFlags,
            pWnd->nAutoSaveInterval);

        WriteProfileInt(SEC_SETTINGS, INI_GBLOPTS,
            CVMEditor::GlobalOpts.lOperation);
        WriteProfileInt(SEC_SETTINGS, INI_ENTMATCH,
            CVMEditor::GlobalOpts.nEnterMatch);
        WriteProfileInt(SEC_SETTINGS, INI_TABSIZE,
            CVMEditor::GlobalOpts.nTabSize);
        WriteProfileInt(SEC_SETTINGS, INI_INDENTSIZE,
            CVMEditor::GlobalOpts.nIndentSize);
        WriteProfileInt(SEC_SETTINGS, INI_RIGHTMARGIN,
            CVMEditor::GlobalOpts.nRightMargin);
        WriteProfileInt(SEC_SETTINGS, INI_SEARCHOPTS,
            CVMEditor::GlobalOpts.nSearch);
        WriteProfileInt(SEC_SETTINGS, INI_SLSTART,
            CVMEditor::GlobalOpts.nInitThreshold);
        WriteProfileInt(SEC_SETTINGS, INI_SLINTERVAL,
            CVMEditor::GlobalOpts.nReportInterval);
        WriteProfileString(SEC_SETTINGS, INI_DEFEXT,
            CVMEditor::GlobalOpts.szDefaultExt);
        WriteProfileString(SEC_SETTINGS, INI_BKEXT,
            CVMEditor::GlobalOpts.szBackupExt);

        WriteProfileInt(SEC_SETTINGS, INI_SHOWGUTTER, CMFCEditor::nDefGutter);
        WriteProfileInt(SEC_SETTINGS, INI_USEAUTOEOL,
            optsDlg.m_Startup.m_bUseAutoEOL);
        WriteProfileInt(SEC_SETTINGS, INI_EDITBUFSIZE,
            optsDlg.m_Startup.m_nEditBufSize);
        WriteProfileInt(SEC_SETTINGS, INI_MAXALLOCATE,
            optsDlg.m_Startup.m_nMaxEditMem);

        WriteProfileInt(SEC_SETTINGS, INI_AUTOSAVEFLAGS,
            pWnd->nAutoSaveFlags);
        WriteProfileInt(SEC_SETTINGS, INI_AUTOSAVEINTERVAL,
            pWnd->nAutoSaveInterval);
    }

    // Update auto save timer
    pWnd->SetAutoSave(TRUE);
}

void CMFCSDIApp::OnDemoTextsize()
{
    CFrameWnd* pFrame = dynamic_cast<CFrameWnd*>(m_pMainWnd);
    if(!pFrame)
        return;

    CMFCEditorView *pView =
        dynamic_cast<CMFCEditorView*>(pFrame->GetActiveView());
    if(!pView)
        return;

    char szMsg[80];

#if defined(CVME_DIAG)
    // This a diagnostic check to see how many characters are in the editor
    // buffer.  It is mainly for debugging the TextSize code, but also returns
    // the LRU counter to see what kind of load the swap mechanism is under.

    int  x;
    long y;

    CVirtualMemory *buf = pView->editor->Buffer;
    pView->editor->textLimits(x, y);

    // V(irtual buffer) + L(ine count) should always equal E(ditor)
    // when efUseLFOnly is disabled.  When efUseLFOnly is enabled,
    // V and E should always be equal.
    sprintf(szMsg, "Diagnostic - Size Info:\nV=%7ld  L=%7ld\n"
        "V+L=%7ld  E=%7ld\nLRU=%d", buf->getTextSize(), y,
        buf->getTextSize() + y, pView->editor->getTextSize(),
        buf->getLRUCnt());

    MessageBox(m_pMainWnd->m_hWnd, szMsg, "Diagnostic Information",
        MB_ICONINFORMATION | MB_OK);
#endif

    sprintf(szMsg, "Selected text size = %ld",
        pView->editor->getSelectedTextSize('\r'));
    MessageBox(m_pMainWnd->m_hWnd, szMsg, "Selected Text",
        MB_ICONINFORMATION | MB_OK);
}

void CMFCSDIApp::OnDemoSetlinedemo()
{
    CSetLineDlg setLineDlg;
    setLineDlg.DoModal();
}

void CMFCSDIApp::OnDemoDefkeymap()
{
	CShowKeyMapDlg showDlg;
    showDlg.DoModal();
}

void CMFCSDIApp::OnDemoMemodlg()
{
	CMemoDemoDlg memoDlg;

    // Set memo text
    memoDlg.m_csMemoOne = m_csFirstMemo;
    memoDlg.m_csMemoTwo = m_csSecondMemo;
    if(memoDlg.DoModal() == IDOK)
    {
        // Get memo text
        m_csFirstMemo = memoDlg.m_csMemoOne;
        m_csSecondMemo = memoDlg.m_csMemoTwo;
    }
}

int CMFCSDIApp::ExitInstance()
{
    // Make sure that any user-created accelerator tables are
    // destroyed.
    for(int nIdx = 0; nIdx < maxKeyMaps; nIdx++)
        CVMEditor::changeKeyMap(nIdx, NULL);

	return CWinApp::ExitInstance();
}
