// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "MFCSDI.h"

#include "MFCEditorDoc.h"
#include "MFCEditorView.h"

#include "MainFrm.h"

#include "CVMEResource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_ACTIVATEAPP()
	ON_WM_CLOSE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP

    // These are here to prevent Wrap On Resize from occurring until
    // the view is all done resizing itself
	ON_WM_SYSCOMMAND()
    ON_MESSAGE(WM_ENTERSIZEMOVE, OnEnterSizeMove)
    ON_MESSAGE(WM_EXITSIZEMOVE, OnExitSizeMove)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
    ID_INDICATOR_CVMSTAT,   // CVMEditor status indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
    ID_INDICATOR_OVR,
    ID_INDICATOR_CVMRO      // CVMEditor Read Only indicator
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
    nAutoSaveFlags = 0;
    nAutoSaveInterval = asDefaultInterval;
    nAutoSaveTimer = 0;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// Delete these three lines if you don't want the toolbar to
	// be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::OnCreateClient( LPCREATESTRUCT /*lpcs*/,
	CCreateContext* pContext)
{
    return m_wndSplitter.Create(this, 2, 2, CSize(2, 2), pContext);
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Modify the Window class or styles here by modifying
	// the CREATESTRUCT cs

	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

// Disable Wrap on Resize until all done
LRESULT CMainFrame::OnEnterSizeMove(WPARAM , LPARAM )
{
    CMFCEditorView *pView = dynamic_cast<CMFCEditorView*>(GetActiveView());
    if(pView)
        pView->WrapOnResize(FALSE);

    return 0;
}

// All done, enable wrap on resize
LRESULT CMainFrame::OnExitSizeMove(WPARAM , LPARAM )
{
    CMFCEditorView *pView = dynamic_cast<CMFCEditorView*>(GetActiveView());
    if(pView)
        pView->WrapOnResize(TRUE);
    return 0;
}

void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam)
{
	CFrameWnd::OnSysCommand(nID, lParam);

    // Force wrap on resize when child window is maximized or restored
    if(nID == SC_MAXIMIZE || nID == SC_ZOOM || nID == SC_RESTORE)
    {
        CMFCEditorView *pView = dynamic_cast<CMFCEditorView*>(GetActiveView());
        if(pView)
            pView->WrapOnResize(TRUE);
    }
}

// Ask the editors to resynch their files if necessary
void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask)
{
	CFrameWnd::OnActivateApp(bActive, hTask);

    // Iterate through all documents
    POSITION tpos = AfxGetApp()->m_pDocManager->GetFirstDocTemplatePosition();
    while(tpos != NULL)
	{
	    CDocTemplate *pTemplate = AfxGetApp()->m_pDocManager->GetNextDocTemplate(tpos);
    	ASSERT_KINDOF(CDocTemplate, pTemplate);

    	POSITION dpos = pTemplate->GetFirstDocPosition();
	    while(dpos != NULL)
    	{
	    	CMFCEditorDoc *pDoc =
                dynamic_cast<CMFCEditorDoc *>(pTemplate->GetNextDoc(dpos));
            if(pDoc)
                if(bActive)
                {
                    // Ask to resynch when gaining focus
                    pDoc->editor->processCommand(CVM_RESYNCHAPP, FALSE);
                    pDoc->UpdateAllViews(NULL);
                }
                else    // Auto save when losing focus?
                    if(nAutoSaveFlags & asSaveOnSwitch)
                        pDoc->editor->autoSave(NULL, TRUE);
    	}
	}
}

void CMainFrame::SetAutoSave(BOOL bStart)
{
    if(nAutoSaveTimer)
        KillTimer(nAutoSaveTimer);

    if(bStart && (nAutoSaveFlags & asAutoSaveEnabled))
        nAutoSaveTimer = SetTimer(100, nAutoSaveInterval * 60000, NULL);
    else
        nAutoSaveTimer = 0;
}

void CMainFrame::OnClose()
{
    SetAutoSave(FALSE);
	CFrameWnd::OnClose();
}

void CMainFrame::OnTimer(UINT nIDEvent)
{
    static BOOL bSaving = FALSE;

    // If another timer message arrives while we are already
    // auto saving or a dialog is present, ignore it.
    if(bSaving || GetLastActivePopup() != this)
        return;

    // Iterate through all documents and perform an auto save
    bSaving = TRUE;

	POSITION tpos = AfxGetApp()->m_pDocManager->GetFirstDocTemplatePosition();
	while(tpos != NULL)
    {
	    CDocTemplate *pTemplate = AfxGetApp()->m_pDocManager->GetNextDocTemplate(tpos);
		ASSERT_KINDOF(CDocTemplate, pTemplate);

    	POSITION dpos = pTemplate->GetFirstDocPosition();
        while(dpos != NULL)
    	{
        	CMFCEditorDoc *pDoc =
                dynamic_cast<CMFCEditorDoc *>(pTemplate->GetNextDoc(dpos));

            if(pDoc)
                pDoc->editor->autoSave();
    	}
    }

	CFrameWnd::OnTimer(nIDEvent);
    bSaving = FALSE;
}
